'use strict';
const shell = require('shelljs');
shell.config.silent = true;
shell.config.fatal = true;

module.exports = (opts = {}) => {
    if (!opts.src)
        opts.src = './src';

    // Reduce accumulator for git-log
    function accumulateCommits(meta, line) {
        // fields are:
        // commit ID,
        // author timestamp, author,
        // committer timestamp, committer
        const fields = line.split(/\s+/);
        var authored = fields[1] && new Date(fields[1]*1000);
        var committed = fields[3] && new Date(fields[3]*1000);

        meta[fields[0]] = {
            authored: authored,
            author: fields[2],
            committed: committed,
            committer: fields[4], 
        };
        
        return meta;
    }

    // Reduce accumulator for git-diff-tree
    function accumulateFiles(meta, line) {
        const match = line.match(/([A-Z])\s+(.*)/);

        if (match) {
            // Parse a file modification
            if (match[1] === 'D') {
                delete meta.files[match[2]];
            }
            else {
                meta.files[match[2]] =
                    meta.commits[meta.commitid];
            }
        }
        else {
            // Parse a commit ID (preceeds associated file mods)
            meta.commitid = line;
        }
        return meta;
    }
    
    /** executes git pull */
    function pullSync() {
        var result = shell.exec('git pull', {cwd: opts.src});
        if (result.code !== 0) {
            throw new Error('Git pull failed:'+ result.stderr);
        }
    }

    function desync(f) {
        return async function() {
            const args = arguments;
            return new Promise((resolve, reject) => {
                try {
                    resolve(f.apply(args));
                }
                catch(e) {
                    reject();
                }
            });
        };
    }
    
    /** gets git metadata */
    function fileInfoSync() {
        const execOpts = {cwd: opts.src, silent: true};

        // List all commit IDs with timestamps leading to this
        const result1 = shell
              .exec('git log --topo-order  --reverse '+
                    '--pretty=format:"%H %at %aN %ct %cN"',
                    execOpts);
        
        if (result1.code !== 0) {
            throw new Error('Git pull failed:'+ result1.stderr);
        }

        // Parse the results into a map of commit IDs to timestamps
        const commits = result1.stdout
              .split(/\n/)
              .reduce(accumulateCommits, {});

        const commitids = Object.keys(commits);

        // Get the files touched by each commit
        const result2 = shell
              .ShellString(commitids.join("\n"), {silent: true})
              .exec('git diff-tree -r --root --name-status --stdin',
                    execOpts);
        
        if (result2.code !== 0) {
            throw new Error('Git pull failed:'+ result2.stderr);
        }

        // Parse this into a map of files to timestamps
        // of the last change
        const acc = result2
              .stdout
              .split(/\n/)
              .reduce(accumulateFiles, {commits, files:{}});

        return acc.files;
    }

    return {
        pullSync, fileInfoSync,
        pull: desync(pullSync),
        fileInfo: desync(fileInfoSync),
    };
};
