'use strict';
const Metalsmith = require('metalsmith');
const assets = require('metalsmith-assets-improved');
const collections = require('metalsmith-collections');
const layouts = require('metalsmith-layouts');
const markdown = require('metalsmith-markdown');
const permalinks = require('metalsmith-permalinks');
const metalsmithDebug = require('metalsmith-debug');
const discoverPartials = require('metalsmith-discover-partials');
const discoverHelpers = require('metalsmith-discover-helpers');
const winston = require('winston');
const fs = require('fs');
const git = require('./lib/git.js')();
const debug = require('debug')('metalsmith.js');
const config = require('./lib/config.js');
const buildlog = require('./lib/logging.js').loggers.build;

/** Iterates over the files, applying func to each */
function forFiles(onFile) {
    return function(files, metalsmith) {
        for(const file in files) {
            const data = files[file];
            // console.log(file, data)
            onFile(data, file, metalsmith);
        }
    };
}

/** Iterates over the files, adding metadata which isn't already set */
function addMeta(meta) {
    return forFiles((data, file) => {
        for(const key in meta) {
            if (key in data)
                continue;

            var val = meta[key];
            if (typeof(val) === 'function')
                val = val(data, file);
            if (val !== undefined)
                data[key] = val;
        }
    });
}

/** Converts a path to a title, so titles don't need to be defined */
function pathToTitle(path) {
    return path
        .split('/')
        .pop()
        .replace(/[.][^.]*/, '')
        .replace(/^(.)/, (_, ch) => ch.toUpperCase())
        .replace(/-(.?)/g, (_, ch) => ' '+ch.toUpperCase());
}

/** Extract a layout name if the file is part of a collection, and an
 * appropriate one exists */
function layoutFromCollection(data, opts = {}) {
    if (!opts.extension)
        opts.extension = '.hbs';
    if (!opts.layoutsDir)
        opts.layoutsDir = 'layouts';
    
    if (data.collection instanceof Array &&
        data.collection.length > 0) {
        // Get the first collection item, ignore the others.
        var layout = `${data.collection[0]}${opts.extension}`;

        // Only use it if there is a corresponding layout file
        if (fs.existsSync(`${__dirname}/${opts.layoutsDir}/${layout}`))
            return layout;
    }
}

const gitMeta = git.fileInfoSync();

// Dummy metadata to omit voluminous/circular crap fields
function elideMeta(meta) {
    return {
            ...meta,
        contents: '...omitted',
        next: meta.next && meta.next.path,
        previous: meta.previous && meta.previous.path,
    };
}

// We return a function building the metalsmith object.
// This works around a metalsmith bug when run in a persistent environment.
// https://github.com/segmentio/metalsmith-collections/issues/27
// Another solution here:
// https://destinmoulton.com/blog/2018/metalsmith-collections-solving-the-duplication-bug/
module.exports = () => {
    return Metalsmith(__dirname)
    .metadata({
        // These defaults can be overridden in pages
        sitename: "Social Coop",
        siteurl: "https://social.coop/",
        wikiurl: "https://git.coop/social.coop/general/wikis/",
        description: "A cooperatively-run corner of the fediverse.",
        generatorname: "Metalsmith",
        generatorurl: "http://metalsmith.io/",
    })
    .ignore([
        '**/*~',  // Emacs backup files
        '.*',     // dotfiles
    ])
    .clean(true)                // clean destination before building
    .use(metalsmithDebug())
    .use(collections({          // Group all blog posts by internally
        posts: {
            pattern: 'posts/*.md', // adding key 'collections':'posts'
        },
    }))
    .use(markdown())
    .use(addMeta({
        title: (d) => pathToTitle(d.path),
        layout: (d) => layoutFromCollection(d),
        filename: (d, f) => f,
        git: (d) => gitMeta[d.path],
    }))
    .use(discoverPartials({
        directory: 'partials',
        pattern: /\.hbs$/
    }))
    .use(discoverHelpers({
        directory: 'helpers',
        pattern: /\.js$/
    }))
    .use(layouts({
        default: 'default.hbs',
    }))
    .use(forFiles((d,f) => buildlog.silly(f,elideMeta(d))))
    .use(assets());
};

if (require.main === module) {
    // Called directly, build
    module.exports().build(function(err) {
        if (!err) return;
        console.error('Build failed: '+err);
        process.exit(1);
    });
}

