'use strict';
const express = require('express');
const fs = require('fs');
const metalsmith = require('./metalsmith.js');
const email = require('./lib/email.js');
const git = require('./lib/git.js')({});
const loggers = require('./lib/logging.js').loggers;
const config = require('./lib/config.js');

const buildlog = loggers.build;
const httplog = loggers.http;
const hooklog = loggers.hook;
const submitlog = loggers.submit;
const maillog = loggers.mail;

const send = email(config.email.server);
const app = express();
const port = config.port;


var expressOptions = {
    dotfiles: 'ignore',
    etag: false,
    //  extensions: ['htm', 'html'],
    index: false,
    maxAge: '1d',
    redirect: false,
    setHeaders: function (res, path, stat) {
        res.set('x-timestamp', Date.now());
    }
};

var fileOptions = {
    root: __dirname + '/build/',
    dotfiles: 'deny',
    headers: {
        'x-timestamp': Date.now(),
        'x-sent': true,
    }
};


async function build() {
    buildlog.info('starting build...')
    metalsmith().build(function(err) {
        if (err)
            buildlog.error('build exception', err);
    })
    buildlog.info('completed build.')
}

app.use(express.static('/', expressOptions));
app.use(express.json());
app.use(express.urlencoded({extended: true}));

app.all('*', (req, res, next) => {
    next();
    httplog.info(`${req.method} ${req.url} ${req.ip} ${res.statusCode}`, {
        method: req.method,
        url: req.url,
        ip: req.ip,
        code: res.code,
    });
})
app.get('/', (req, res) => {
    res.redirect('/home.html');
});
app.get('/style.css', (req, res) => {
    res.sendFile(req.path, fileOptions);
});
app.get('*', (req, res, next) => {
    var path = req.path+'/index.html';
    if (fs.existsSync(fileOptions.root+path)) {
        // Cater to the wiki's nested-indexes style
        res.sendFile(path, fileOptions);
        return;
    }
    if (fs.existsSync(fileOptions.root+req.path)) {
        res.sendFile(req.path, fileOptions);
        return;
    }
    path = req.path+'.html';
    if (fs.existsSync(fileOptions.root+path)) {
        res.redirect(path);
        return;
    }
    next();
});

if (!config.webhook) {
    console.log("No webhook configured!");
}
else{
    app.post(config.webhook.path, (req, res) => {
        var secret = req.header('X-Gitlab-Token');
        if (secret !== config.webhook.secret) {
            res.sendStatus(403); // Unauthorised
            hooklog.warn(`rejecting unauthorised hook call`);
            return;
        }
        
        try {
            hooklog.info(`handling hook call by pulling`);
            git // Async
                .pull()
                .then(build)
                .then(() => hooklog.info(`handling hook call complete`));
        }
        catch(e) {
            hooklog.error('error handling hook', e);
        }
        res.sendStatus(200);
    });
}
app.post('/registration', (req, res) => {
    function ok() {}
    function error(err) {
        submitlog.info("error emailing registration notification, ",
                       err);
    }
    
    submitlog.info('emailing registration notification: ', req.body);
    const sendConfig = config.email.registration || {};
    send(sendConfig,
         `<pre>${JSON.stringify(req.body, null, 2)}</pre>`)
        .then(ok, error);

    res.redirect('/home');
});

app.listen(port, () => console.log(`serve.js listening on port ${port}!`));
